<?php
/**
 * A general file-based stack class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2013 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class BaseFileStack extends BaseStacker {
	/**
	 * Magic for this stack
	 */
	const STACK_MAGIC = 'STACKv0.1';

	/**
	 * Separator magic->count
	 */
	const SEPARATOR_MAGIC_COUNT = 0x00;

	/**
	 * Separator position->entries
	 */
	const SEPARATOR_SEEK_POS_ENTRIES = 0xff;

	/**
	 * Separator hash->name
	 */
	const SEPARATOR_HASH_NAME = 0x05;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Checks whether the file header is initialized
	 *
	 * @return	$isInitialized	Whether the file header is initialized
	 */
	private function isFileHeaderInitialized () {
		// Default is not initialized
		$isInitialized = FALSE;

		// Is the file initialized?
		if ($this->isFileInitialized()) {
			// Some bytes has been written, so rewind to start of it.
			$this->getIteratorInstance()->rewind();

			// Read file header
			$this->readFileHeader();
		} // END - if

		// Return result
		return $isInitialized;
	}

	/**
	 * Checks whether the file-based stack has been initialized
	 *
	 * @return	$isInitialized		Whether the file's size is zero
	 */
	private function isFileInitialized () {
		// Default is not initialized
		$isInitialized = FALSE;

		// Get it from iterator which holds the pointer instance. If FALSE is returned
		$fileSize = $this->getIteratorInstance()->size();

		/*
		 * The returned file size should not be FALSE or NULL as this means
		 * that the pointer class does not work correctly.
		 */
		assert(is_int($fileSize));

		// Is more than 0 returned?
		if ($fileSize > 0) {
			// So is the header written?
			$isInitialized = $this->getIteratorInstance()->isHeaderInitialized();
		} // END - if

		// Return result
		return $isInitialized;
	}

	/**
	 * Initializes this file-based stack.
	 *
	 * @param	$fileName	File name of this stack
	 * @return	void
	 */
	protected function initFileStack ($fileName) {
		// Get a file i/o pointer instance
		$pointerInstance = ObjectFactory::createObjectByConfiguredName('file_raw_input_output_class', array($fileName));

		// Get iterator instance
		$iteratorInstance = ObjectFactory::createObjectByConfiguredName('file_io_iterator_class', array($pointerInstance));

		// Is the instance implementing the right interface?
		assert($iteratorInstance instanceof SeekableWritableFileIterator);

		// Set iterator here
		$this->setIteratorInstance($iteratorInstance);

		// Is the file's header initialized?
		if ($this->isFileHeaderInitialized()) {
			// Then load it
			$this->loadFileHeader();
		} else {
			// No, then create it (which may pre-allocate the stack)
			$this->createFileHeader();
		}
	}

	/**
	 * Initializes given stacker
	 *
	 * @param	$stackerName	Name of the stack
	 * @param	$forceReInit	Force re-initialization
	 * @return	void
	 * @throws	AlreadyInitializedStackerException	If the stack is already initialized
	 */
	public function initStack ($stackerName, $forceReInit = FALSE) {
		// Is the stack already initialized?
		if (($forceReInit === FALSE) && ($this->isStackInitialized($stackerName))) {
			// Then throw the exception
			throw new AlreadyInitializedStackerException(array($this, $stackerName, $forceReInit), self::EXCEPTION_STACKER_ALREADY_INITIALIZED);
		} // END - if

		// Initialize the given stack
		$this->partialStub('stackerName=' . $stackerName . ',forceReInit=' . intval($forceReInit));
	}

	/**
	 * Checks whether the given stack is initialized (set in array $stackers)
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$isInitialized	Whether the stack is initialized
	 */
	public function isStackInitialized ($stackerName) {
		// Is is there?
		$this->partialStub('stackerName=' . $stackerName);
		$isInitialized = TRUE;

		// Return result
		return $isInitialized;
	}

	/**
	 * Getter for size of given stack (array count)
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$count			Size of stack (array count)
	 * @throws	NoStackerException	If given stack is missing
	 */
	public function getStackCount ($stackerName) {
		// Is the stack not yet initialized?
		if (!$this->isStackInitialized($stackerName)) {
			// Throw an exception
			throw new NoStackerException(array($this, $stackerName), self::EXCEPTION_NO_STACKER_FOUND);
		} // END - if

		// Now, count the array of entries
		$this->partialStub('stackerName=' . $stackerName);
		$count = 0;

		// Return result
		return $count;
	}

	/**
	 * Adds a value to given stack
	 *
	 * @param	$stackerName	Name of the stack
	 * @param	$value			Value to add to this stacker
	 * @return	void
	 * @throws	FullStackerException	Thrown if the stack is full
	 */
	protected function addValue ($stackerName, $value) {
		// Is the stack not yet initialized or full?
		if (!$this->isStackInitialized($stackerName)) {
			// Then do it here
			$this->initStack($stackerName);
		} elseif ($this->isStackFull($stackerName)) {
			// Stacker is full
			throw new FullStackerException(array($this, $stackerName, $value), self::EXCEPTION_STACKER_IS_FULL);
		}

		// Now add the value to the stack
		$this->partialStub('stackerName=' . $stackerName . ',value[]=' . gettype($value));
	}

	/**
	 * Get last value from named stacker
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$value			Value of last added value
	 * @throws	NoStackerException	If the named stacker was not found
	 * @throws	EmptyStackerException	If the named stacker is empty
	 */
	protected function getLastValue ($stackerName) {
		// Is the stack not yet initialized or full?
		if (!$this->isStackInitialized($stackerName)) {
			// Throw an exception
			throw new NoStackerException(array($this, $stackerName), self::EXCEPTION_NO_STACKER_FOUND);
		} elseif ($this->isStackEmpty($stackerName)) {
			// Throw an exception
			throw new EmptyStackerException(array($this, $stackerName), self::EXCEPTION_STACKER_IS_EMPTY);
		}

		// Now get the last value
		$this->partialStub('stackerName=' . $stackerName);
		$value = NULL;

		// Return it
		return $value;
	}

	/**
	 * Get first value from named stacker
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$value			Value of last added value
	 * @throws	NoStackerException	If the named stacker was not found
	 * @throws	EmptyStackerException	If the named stacker is empty
	 */
	protected function getFirstValue ($stackerName) {
		// Is the stack not yet initialized or full?
		if (!$this->isStackInitialized($stackerName)) {
			// Throw an exception
			throw new NoStackerException(array($this, $stackerName), self::EXCEPTION_NO_STACKER_FOUND);
		} elseif ($this->isStackEmpty($stackerName)) {
			// Throw an exception
			throw new EmptyStackerException(array($this, $stackerName), self::EXCEPTION_STACKER_IS_EMPTY);
		}

		// Now get the first value
		$this->partialStub('stackerName=' . $stackerName);
		$value = NULL;

		// Return it
		return $value;
	}

	/**
	 * "Pops" last entry from stack
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$value			Value "poped" from array
	 * @throws	NoStackerException	If the named stacker was not found
	 * @throws	EmptyStackerException	If the named stacker is empty
	 */
	protected function popLast ($stackerName) {
		// Is the stack not yet initialized or full?
		if (!$this->isStackInitialized($stackerName)) {
			// Throw an exception
			throw new NoStackerException(array($this, $stackerName), self::EXCEPTION_NO_STACKER_FOUND);
		} elseif ($this->isStackEmpty($stackerName)) {
			// Throw an exception
			throw new EmptyStackerException(array($this, $stackerName), self::EXCEPTION_STACKER_IS_EMPTY);
		}

		// Now, remove the last entry, we don't care about the return value here, see elseif() block above
		$this->partialStub('stackerName=' . $stackerName);
		return NULL;
	}

	/**
	 * "Pops" first entry from stack
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$value			Value "shifted" from array
	 * @throws	NoStackerException	If the named stacker was not found
	 * @throws	EmptyStackerException	If the named stacker is empty
	 */
	protected function popFirst ($stackerName) {
		// Is the stack not yet initialized or full?
		if (!$this->isStackInitialized($stackerName)) {
			// Throw an exception
			throw new NoStackerException(array($this, $stackerName), self::EXCEPTION_NO_STACKER_FOUND);
		} elseif ($this->isStackEmpty($stackerName)) {
			// Throw an exception
			throw new EmptyStackerException(array($this, $stackerName), self::EXCEPTION_STACKER_IS_EMPTY);
		}

		// Now, remove the last entry, we don't care about the return value here, see elseif() block above
		$this->partialStub('stackerName=' . $stackerName);
		return NULL;
	}
}

// [EOF]
?>
